#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""add hidden to cluster template

Revision ID: 87e62e3c7abc
Revises: cbbc65a86986
Create Date: 2019-02-05 15:35:26.290751

"""

# revision identifiers, used by Alembic.
revision = '87e62e3c7abc'
down_revision = 'cbbc65a86986'

from alembic import op  # noqa: E402
import sqlalchemy as sa  # noqa: E402


def upgrade():
    op.add_column('cluster_template',
                  sa.Column('hidden',
                            sa.Boolean(create_constraint=False),
                            default=False))
